=begin
#-----------------------------------------------------------------------
Copyright 2007 TIG
Permission to use, copy, modify, and distribute this software for 
any purpose and without fee is hereby granted, provided something the 
above copyright notice appear in all copies.
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------
Name        : 	ComponentReporter+.rb#
Type        : 	Tool
Description : 	Makes a report all components placed in a model.
Menu Item   : 	Plugins >> Component+ Report
Usage       : 	Pick and a report is written into a file called 
				XXX-ComponentReport+.csv for the Component Definitions.
				another report called XXX-Instance+Report.csv list info
				on individual palced components is also made.
				The reports are found in the model's folder.  
				They are readable by Excel and similar programs.
Version     :	1.0 20071022	First Issue.
				1.1	20071023	Reports instances in selection too.
#-----------------------------------------------------------------------
=end
require 'sketchup.rb'
### do class
class Reporter
### do def
 def Reporter::components_plus
   model=Sketchup.active_model
   entities=model.entities
   mname=model.title
   ss=model.selection###v1.1
### start undo...
   model.start_operation("Component+ Report")
### show VCB and status info...
   Sketchup::set_status_text(("COMPONENT+ REPORTER..." ), SB_PROMPT)
   Sketchup::set_status_text(" ", SB_VCB_LABEL)
   Sketchup::set_status_text(" ", SB_VCB_VALUE)
### check model...
   mpath=Dir::pwd
   if mpath==""
      UI.messagebox("You must save the 'Untitled' new model \nbefore making a Component+ Report !\nExiting... ")
      return nil
   end
### setup cdefn list...
   clist=[]
   model.definitions.each{|c|
      if c.count_instances > 0 and not c.group?
        clist.push([c.name,c.count_instances.to_s,c.description,c.guid])
      end#if
   } 
   clist.sort!
### if no components exit...
   if not clist[0]
      UI.messagebox("No Components available to make a Component+ Report !\nExiting... ")
      return nil
   end#if
### write to csv file...
   ccsv=mpath + "/" + mname + "-Component+Report.csv"
   file=File.new(ccsv,"w")
   file.puts("DEF-NAME,NUMBER,DESCRIPTION,GUID\n\n") 
   ### title ### add what you want to list here
   clist.each{|c|
     file.puts(c[0]+","+c[1]+","+c[2]+","+c[3])
   }
   file.close
### check you really want to do compos in selection
### but must check if ss has compo instances in it.
   compo_selected=false
   ss.each{|c|compo_selected=true if c.typename=="ComponentInstance"  and not c.definition.group?}
   sel=""
   if compo_selected
      if UI.messagebox("Do you want the Instance Report to be based on just the Selection ?  ",MB_YESNO,"Just Selection ?")==6 ### 6=YES 7=NO
	    entities=ss
	    sel=" (for Selection)"
	  end#if
   end#if
### NOW make a list of all "instances" with details
### setup instances list...
   inlist=[]
   entities.each{|c|
      inlist.push(c) if c.typename=="ComponentInstance" and not c.definition.group?
   } 
   ilist=[]
   inlist.each{|c|
     dname="'"+c.definition.name+"'"###avoids loss of front zeros
	 iname="'"+c.name+"'"###avoids loss of front zeros
	 lname="'"+c.layer.name+"'"###avoids loss of front zeros
	 mat=c.material
	 if mat
	   cname="'"+mat.name+"'"###avoids loss of front zeros
	 else
	   cname="'<Default>'"###avoids loss of front zeros
	 end#if
	 area=0.0
	 c.definition.entities.each{|e|area=area+e.area if e.typename=="Face"}
	 area=area.to_s
	 ### this is in sq inches ### make a tweak here to suit your units
	 ### e.g. area=(area/144).to_s for sq ft
	 id=c.entityID.to_s
	 x=(c.bounds.max.x-c.bounds.min.x).to_l.to_s
	 y=(c.bounds.max.y-c.bounds.min.y).to_l.to_s
	 z=(c.bounds.max.z-c.bounds.min.z).to_l.to_s
     ilist.push([dname,iname,lname,cname,area,id,x,y,z])
   }
   ilist.sort!
### write to csv file...
   icsv=mpath + "/" + mname + "-Instance+Report.csv"
   file=File.new(icsv,"w")
   file.puts("DEF-NAME,INST-NAME,LAYER,MATERIAL,AREA,ID,X,Y,Z\n\n") 
   ### title ### add what you want to list here
   ilist.each{|c|
     file.puts(c[0]+","+c[1]+","+c[2]+","+c[3]+","+c[4]+","+c[5]+","+c[6]+","+c[7]+","+c[8])
   }
   file.close
### say it's done...
   UI.messagebox("Component+ Report written into:\n\n"+ccsv+"\n\nInstance+ Report"+sel+" written into:\n\n"+icsv+"\n\n")
### commit undo...
   model.commit_operation
 end#def components
end#class
### Sketchup.send_action "showRubyPanel:"###
### do menu
if not file_loaded?("ComponentReporter+.rb")
    add_separator_to_menu("Plugins")
    UI.menu("Plugins").add_item("Component+ Report"){Reporter.components_plus}
end#if
file_loaded("ComponentReporter+.rb")
###
